#!/bin/sh

 #######################################################################
 # Copyright 2007-2009 Igor Daniloff.
 # 
 # Following source code is the property of Doctor Web, Ltd.
 # Use is subject to license terms.
 # The reproduction in any form without written permission of
 # Doctor Web, Ltd. and proper attribution is strongly prohibited.
 #######################################################################


#vars 

if [ -f /usr/xpg4/bin/awk ]; then
    AWK=/usr/xpg4/bin/awk
else
    AWK=awk
fi

if [ -f /usr/xpg4/bin/sed ]; then
    SED=/usr/xpg4/bin/sed
else
    SED=sed
fi

if [ -f /usr/xpg4/bin/grep ]; then
    GREP=/usr/xpg4/bin/grep
else
    GREP=grep
fi

if [ -f /usr/xpg4/bin/egrep ]; then
    EGREP=/usr/xpg4/bin/egrep
else
    EGREP=egrep
fi

DEBUG="no"
#ROOT_PATH=""
DEFAULT_BIN_PATH=/opt/drweb
DEFAULT_BIN_PATH=`echo "$DEFAULT_BIN_PATH" | $SED 's/^\///'`
SCRIPTS_DIR="$DEFAULT_BIN_PATH/maild/scripts"
DOC_DIR="$DEFAULT_BIN_PATH/doc/maild"
DEFAULT_VAR_PATH="$ROOT_PATH/var/drweb"
if [ "0" = "1" ]; then
   INIT_SCRIPT="$ROOT_PATH/usr/local/etc/rc.d/00.drweb-monitor.sh"
   SYS_LIB_DIR="/usr/local/lib"
   DW_LIB_DIR="$ROOT_PATH/usr/local/drweb/lib"
else
   INIT_SCRIPT="$ROOT_PATH/etc/init.d/drweb-monitor"
   SYS_LIB_DIR="/usr/local/lib"
   DW_LIB_DIR="$ROOT_PATH/opt/drweb/lib"
fi

#functions
######################################################################
dwd()
{
    if [ "$DEBUG" = "yes" ]; then
        echo $1
    fi
}
######################################################################
DwFindFullPath()
{
    if [ -x "./$0" ] ; then
        var=`echo $0`
        var=`echo "$var" | $SED 's/^\.\///'`
        SCRIPT_PATH=`pwd`/"$var"
        SCRIPT_PATH=`echo "$SCRIPT_PATH" | $SED 's/\"//g'`
    else
        SCRIPT_PATH="$0"
    fi
    SCRIPT_NAME=`echo "$SCRIPT_PATH" | $SED 's/^.*\/\([^\/]*\)$/\1/'`
    SCRIPT_PATH=`echo "$SCRIPT_PATH" | $SED 's/\/[^\/]*$//'`
    dwd "SCRIPT_PATH=$SCRIPT_PATH SCRIPT_NAME=$SCRIPT_NAME"
}

######################################################################
DwInstall()
{
(cd "$1"
    CHOWN=""
    ADD="no"
    for element in *; do
        if [ -d "$element" ]; then
            if [ ! -d "$2/$element" ]
            then
                if [ "$3" = "modif" -o "$3" = "update" ]
                then
                    mkdir -p "$2/$element" || exit 1
                    chown drweb:drweb "$2/$element" || exit 1
                    chmod 0775 "$2/$element" || exit 1
                    ADD="yes"
                    dwd "$element => $2/$element"
                fi
            fi
            DwInstall $element "$2/$element" $3 || exit 1
            if [ "$ADD" = "yes" ]
            then
                echo "$2/$element" >> "$DW_INSTALL_LOG"
            fi
        fi
        suffix=""
        if [ -f "$element"  -a "$element" != "$SCRIPT_NAME"  -a "$element" != "update.sh" ]; then
            ext=`echo "$element" | $SED 's/^.*\.\([^\.]*\)$/\1/'`
            if [ -f "$2/$element" ]; then
                if [ "$element" = "emails" -o "$ext" = "conf" -o "$ext" = "ini" -o "$ext" = "mmc" ]; then
                    if [ "$3" = "modif" ]
                    then
                        echo "File $2/$element exist: save it to $2/$element.drwebsave"
                        mv "$2/$element" "$2/$element.drwebsave" || exit 1
                    elif [ "$3" = "update" ]
                    then
                        echo "File $2/$element exist: save new version to $2/$element.new"
                        suffix=".new"
                    fi
                elif [ "$3" != "update" ]; then
                    echo "File $2/$element already exist: can not continue!"
                    exit 1
                fi
            fi
            if [ "$3" != "fake" ]
            then
                if [ ! -f "$2/$element$suffix" ]; then
                    echo "$2/$element$suffix" >> "$DW_INSTALL_LOG"
                    dwd "$element ($ext) => $2/$element$suffix"
                fi
                if [ -L "$element" ]; then
                    STAT_INFO=`ls -l "$element" | $SED "s/^.*-> \(.*\)$/\1/"`
                    (cd "$2"; if [ -f "$element$suffix" ]; then rm -f "$element$suffix"; fi; ln -s "$STAT_INFO" "$element$suffix"|| exit 1) || exit 1
                else
                    cp -p -R "$element" "$2/$element$suffix" || exit 1
                fi
                CHOWN="$CHOWN $2/$element$suffix"
            fi
        fi
    done
    if [ ! -z "$CHOWN" ]; then
        chown drweb:drweb $CHOWN || exit 1
    fi
)
}
######################################################################
DwStop()
{
    if [ ! -f $INIT_SCRIPT ]; then
        echo "Can not find init script $INIT_SCRIPT"
        exit 2
    fi
    
    $INIT_SCRIPT status
    if [ $? -eq 0 ]; then
        $INIT_SCRIPT stop
        if [ $? -ne 0 ]; then
            echo "Can not stop Dr.Web MailD"
            exit 3
        fi
    fi
}
######################################################################
DwReload()
{
    if [ -x $INIT_SCRIPT ]; then
        $INIT_SCRIPT status
        if [ $? -eq 0 ]; then
            $INIT_SCRIPT reload
            if [ $? -eq 0 ]; then
                echo "===> Dr.Web for Unix mail servers reloaded"
            else
                echo "" 
                echo "Automatic reload failed."
                echo "Please inspect Dr.Web for Unix mail servers and plugins configuration for default values."
                echo "Please start drweb-monitor manually."
            fi
        fi
    fi
}
######################################################################
DwRestart()
{
    if [ -x $INIT_SCRIPT ]; then
        $INIT_SCRIPT status
        if [ $? -eq 0 ]; then
            $INIT_SCRIPT restart
            if [ $? -eq 0 ]; then
                echo "===> Dr.Web for Unix mail servers restarted"
            else
                echo "" 
                echo "Automatic start failed."
                echo "Please inspect Dr.Web for Unix mail servers configuration for default values."
                echo "Please start drweb-monitor manually."
            fi
        fi
    elif [ -x "`which svcadm`" ] && svcs drweb-monitor >/dev/null 2>&1; then
        # Would not restart if disabled currently.
        svcadm restart drweb-monitor
    else
        echo ""
        echo "Dr.Web for Unix mail servers restart failed."
        echo "Neither SysV nor SVC service was found on your system."
    fi
}

######################################################################
DwBackup()
{
    if [ -f "$1" ] ; then

        if [ -f "$1.drwebsave" ] ; then

            CONF_DRWEBSAVE="yes"
            TIMESTAMP=`date "+%Y%m%d-%k_%M_%S"`
            echo ""
            echo "Info: $1.drwebsave exists. Saving $1 as $1.$TIMESTAMP"
            echo ""
            cp "$1" "$1.$TIMESTAMP"
            if [ $? -ne 0 ] ; then
                echo ""
                echo "Error: can't backup $1! Installation aborted."
                echo ""
                return 3
            fi
        fi
        if [ "$CONF_DRWEBSAVE" != "yes" ] ; then
            cp "$1" "$1.drwebsave"
            if [ $? -ne 0 ] ; then
                echo ""
                echo "Error: can't backup $1! Installation aborted."
                echo ""
                return 2
            else
                echo ""
                echo "Success: backup of $1 in $1.drwebsave."
                echo ""
            fi

        fi
    else
        echo ""
        echo "Error: can't find $1!"
        echo ""
        return 1
    fi
    return 0;
}
######################################################################
DwPostProc()
{
    chmod +s "$ROOT_PATH/$DEFAULT_BIN_PATH/drweb-inject"
}

######################################################################
DwInstallLibs()
{
:
#     for file in libstdc++.so.6 libgcc_s.so.1
#     do
#       if [ ! -f $SYS_LIB_DIR/$file ]; then
#           dwd "Make link from $DW_LIB_DIR/$file to $SYS_LIB_DIR/$file"
#           rm -f $SYS_LIB_DIR/$file >/dev/null 2>&1
#           ln -s $DW_LIB_DIR/$file $SYS_LIB_DIR/$file || exit 1
#       fi.
#     done
}
######################################################################
DwSetPluginName()
{
    OLD=`pwd`
    if [ ! -d "/$PLUGINS_DIR" ];then
        echo "Can not find plugin directory '/$PLUGINS_DIR'"
        exit 1
    fi
    cd "/$PLUGINS_DIR"
    for plugin in *; do
        PLUGIN_NAME=`echo "$plugin" | $SED 's/^lib\(.*\)\.so$/\1/'`
        dwd "PLUGIN_NAME = $PLUGIN_NAME"
        break
    done
    cd "$OLD"
    if [ "x$PLUGIN_NAME" = "x*" ]; then
        echo "Can not find any plugin library in /$PLUGINS_DIR"
        exit 1
    fi
}

DwConfigureMonitor()
{
    DEFAULT_CONF_PATH=/etc/drweb
    if test -f "$DEFAULT_CONF_PATH/monitor.conf" ; then
        RES=`cat $DEFAULT_CONF_PATH/monitor.conf | $EGREP "^User = root$"| head -n1`
        if test -z "$RES" ; then
	    MONITOR_CONF_SAVE="$DEFAULT_CONF_PATH/monitor.conf".drwebsave-`date +%Y%m%d_%H-%M-%S`
	    cp "$DEFAULT_CONF_PATH/monitor.conf" "$MONITOR_CONF_SAVE"
            $SED "s/User = drweb/User = root/" "$MONITOR_CONF_SAVE" > "$DEFAULT_CONF_PATH/monitor.conf"
            RES=`cat $DEFAULT_CONF_PATH/monitor.conf | $EGREP "^User = root$"| head -n1`
            if test -n "$RES" ; then
                echo "$DEFAULT_CONF_PATH/monitor.conf patched."
                if test -z $POSTINSTALL_FLAG; then
                    DwRestart
                fi
            else
                echo "$DEFAULT_CONF_PATH/monitor.conf not patched. Change User = drweb to User = root manually if needed."
            fi
        else
            echo "$DEFAULT_CONF_PATH/monitor.conf already patched."
        fi
    fi
}

DwConfigureMonitorRunAppList()
{
    DEFAULT_CONF_PATH=/etc/drweb
    if test -f "$DEFAULT_CONF_PATH/monitor.conf" ; then
        RES=`cat $DEFAULT_CONF_PATH/monitor.conf | $EGREP "^RunAppList = MAILDESK$"| head -n1`
        if test -z "$RES" ; then
	    MONITOR_CONF_SAVE="$DEFAULT_CONF_PATH/monitor.conf".drwebsave-`date +%Y%m%d_%H-%M-%S`
	    cp "$DEFAULT_CONF_PATH/monitor.conf" "$MONITOR_CONF_SAVE"
            $SED "s/RunAppList = MAILD/RunAppList = MAILDESK/" "$MONITOR_CONF_SAVE" > "$DEFAULT_CONF_PATH/monitor.conf"
            RES=`cat $DEFAULT_CONF_PATH/monitor.conf | $EGREP "^RunAppList = MAILDESK$"| head -n1`
            if test -n "$RES" ; then
                echo "$DEFAULT_CONF_PATH/monitor.conf patched."
                DwRestart
            else
                echo "$DEFAULT_CONF_PATH/monitor.conf not patched. Change 'RunAppList = MAILD' to 'RunAppList = MAILDESK' manually if needed."
            fi
        else
            echo "$DEFAULT_CONF_PATH/monitor.conf already patched."
        fi
    fi
}

############ PRE ##############

#root

id | $EGREP "uid=0\(" 1> /dev/null
if [ $? -ne 0 ]; then
    echo "You must be root to run the Dr.Web for Unix mail servers installer."
    exit 1
fi

#reset umask
umask 0002

cd "/$SCRIPTS_DIR"

DwFindFullPath


#vars
MTA_CONFIGURE="no"
DW_INSTALL_LOG="$DEFAULT_VAR_PATH/maild_CGP_install.info"

DwPrepareMta()
{
:
}
DwUpdateMTA()
{
:
}
DwInstallMtaCGP()
{

    DwConfigureMonitor
    MAINSETTINGS_CF_PATCHED=""
    RULESSETTINGS_CF_PATCHED=""

    if [ "x$MTA_CONFIG_PATH" != "x" ] ; then
        MAINSETTINGS_CF="$MTA_CONFIG_PATH/Main.settings"
        RULESSETTINGS_CF="$MTA_CONFIG_PATH/Rules.settings"
    else
        MAINSETTINGS_CF="/var/CommuniGate/Settings/Main.settings"
        RULESSETTINGS_CF="/var/CommuniGate/Settings/Rules.settings"
    fi

    if [ -f $MAINSETTINGS_CF ] ; then 
        IF_FILTER=`cat $MAINSETTINGS_CF |$GREP -v "#" |$GREP "DrWeb Maild"`
        if [ "x$IF_FILTER" = "x"  ] ; then
            DwBackup $MAINSETTINGS_CF || return 1
        else
            echo ""
            echo "Info: $MAINSETTINGS_CF already contains Dr.Web Maild filter, skipping patching $MAINSETTINGS_CF."
            MAINSETTINGS_CF_PATCHED="x"
        fi
    else
        echo ""
        echo "Error: can't find $MAINSETTINGS_CF."
        echo "Info: you can specify the MTA_CONFIG_PATH environment variable."
        return 1
    fi

    if [ -f $RULESSETTINGS_CF ] ; then 
        IF_RFILTER=`cat $RULESSETTINGS_CF |$GREP -v "#" |$GREP "DrWeb Maild"`
        if [ "x$IF_RFILTER" = "x"  ] ; then
            DwBackup $RULESSETTINGS_CF || return 1
        else
            echo ""
            echo "Info: $RULESSETTINGS_CF already contains Dr.Web Maild filter, skipping patching $RULESSETTINGS_CF."
            RULESSETTINGS_CF_PATCHED="x"
        fi
    fi

    if [ "$MAINSETTINGS_CF_PATCHED" = "x" -a "$RULESSETTINGS_CF_PATCHED" = "x" ] ; then
        echo "Info: Both $RULESSETTINGS_CF and $MAINSETTINGS_CF seems to be already patched."
        MTA_CONFIGURE="yes"
        return 0
    fi

    IS_EXTFILTER=`cat $MAINSETTINGS_CF |$GREP -i "externalfilters"`
    IS_EXTFILTER_NULL=`cat $MAINSETTINGS_CF |$GREP -i "externalfilters" |$GREP -i ";"`
    STR_FILTER="{ Enabled = YES; LogLevel = 3; Name = \"DrWeb Maild\"; ProgramName = \"$ROOT_PATH/$DEFAULT_BIN_PATH/drweb-cgp-receiver\"; RestartPause =  1m; Timeout =  2m; }"
    if [ "x$IS_EXTFILTER" != "x" ] ; then
	if [ "x$IS_EXTFILTER_NULL" != "x" ] ; then
            IS_LBRACE=`echo $IS_EXTFILTER | grep {`
            if [ "x$IS_LBRACE" = "x" ] ; then
                $SED -e "s|ExternalFilters.*$|ExternalFilters = ( $STR_FILTER );|" $MAINSETTINGS_CF >> $MAINSETTINGS_CF.tmp
            else
                $AWK -F "(" -v STR_FILTER="$STR_FILTER" '{if ($0 !~ /ExternalFilters.*/) print; else print $1 "( " STR_FILTER ", " $2}' $MAINSETTINGS_CF >> $MAINSETTINGS_CF.tmp
            fi
            mv -f $MAINSETTINGS_CF.tmp $MAINSETTINGS_CF
        else
            $AWK -v STR_FILTER="$STR_FILTER" '{print}/ExternalFilters.*/{print "    " STR_FILTER ", "}' $MAINSETTINGS_CF >> $MAINSETTINGS_CF.tmp
            mv -f $MAINSETTINGS_CF.tmp $MAINSETTINGS_CF
        fi
    else
        head -n1 $MAINSETTINGS_CF > $MAINSETTINGS_CF.tmp
        STR="ExternalFilters = ( $STR_FILTER );"
        echo $STR >>  $MAINSETTINGS_CF.tmp
        STRCOUNT=`cat $MAINSETTINGS_CF | wc -l `
        LASTSTRINGS=`echo $STRCOUNT-1 | bc`
        tail -n $LASTSTRINGS $MAINSETTINGS_CF >> $MAINSETTINGS_CF.tmp
        mv $MAINSETTINGS_CF.tmp $MAINSETTINGS_CF
    fi

    if [ -f $RULESSETTINGS_CF ] ; then
        CHECK_RULESSETTINGS_CF=`cat $RULESSETTINGS_CF`
        if [ "$CHECK_RULESSETTINGS_CF" = "()"  ] ; then
            sed -e "s/()/((9, \"DrWeb Maild\", (), ((ExternalFilter, \"DrWeb Maild\"))))/g" $RULESSETTINGS_CF > $RULESSETTINGS_CF.tmp
            mv $RULESSETTINGS_CF.tmp $RULESSETTINGS_CF
        else
            sed -e "$,$ s/)$/,(9, \"DrWeb Maild\", (), ((ExternalFilter, \"DrWeb Maild\"))))/g" $RULESSETTINGS_CF > $RULESSETTINGS_CF.tmp
	    if ( ! grep 'DrWeb' $RULESSETTINGS_CF.tmp > /dev/null ) ; then
		echo "Patch $RULESSETTINGS_CF fail. Rewrite $RULESSETTINGS_CF with DrWeb Maild filter."
        	echo "(" >> $RULESSETTINGS_CF.tmp
        	echo "(9, \"DrWeb Maild\", (), ((ExternalFilter, \"DrWeb Maild\")))" >> $RULESSETTINGS_CF.tmp
        	echo ")" >> $RULESSETTINGS_CF.tmp
        	STRCOUNT_RULE=`cat $RULESSETTINGS_CF | wc -l `
        	LASTSTRINGS_RULE=$STRCOUNT_RULE
        	tail -n $LASTSTRINGS_RULE $RULESSETTINGS_CF >> $RULESSETTINGS_CF.tmp
    	    fi
            mv $RULESSETTINGS_CF.tmp $RULESSETTINGS_CF
        fi
    else
        echo "(" > $RULESSETTINGS_CF
        echo "(9, \"DrWeb Maild\", (), ((ExternalFilter, \"DrWeb Maild\")))" >> $RULESSETTINGS_CF
        echo ")" >> $RULESSETTINGS_CF
    fi

    echo "Info: $MAINSETTINGS_CF and $RULESSETTINGS_CF successfully patched."

    MTA_CONFIGURE="yes"
}



############ START ##############

if [ "x$1" = "xupdate" ] ; then
    echo
    echo ------------------------------------------------------------
    echo     Welcome to the Dr.Web InstallShield Wizard.
    echo
    echo The InstallShield Wizard will update configuration of CGP.
    echo
    while true ; do
        echo "Do you wish to continue?"
        read yesno
        case "$yesno" in
            y | yes | Y | Yes | YES)
            break
            ;;
            n | no | N | No | NO)
            echo "Try later. Bye!"
            exit 1
            ;;
            *)
            echo Please enter yes or no.
            ;;
        esac
    done
    DWPrepareMta
    DwUpdateMTA
fi


DwPrepareMta

#configure MTA
if  [ "x$SKIP_MTA_CONFIG" != "xyes" ]; then
    echo
    echo ------------------------------------------------------------
    echo     Welcome to the Dr.Web InstallShield Wizard.
    echo
    echo The InstallShield Wizard will configure CGP.
    echo
    while true ; do
        echo "Perform MTA configuration?"
	echo "Please enter yes or no."
        read yesno
        case "$yesno" in
            y | yes | Y | Yes | YES)
            break
            ;;
            n | no | N | No | NO)
            echo "Try later. Bye!"
            exit 1
            ;;
            *)
            echo Please enter yes or no.
            ;;
        esac
    done
    DwInstallMtaCGP
fi

if [ "CGP" != "MAILDESK" ]; then
  if [ "$MTA_CONFIGURE" = "no" ]
  then
    echo "Please, refer to documentation on CGP adjustment residing in $ROOT_PATH/$DOC_DIR directory."
  else
    echo "Please, restart CGP manually."
  fi
fi

